<?php
//	Copyright (c) 2009-2015 Daniel Robert Bradley. All rights reserved.
//	This software is distributed under the terms of the GNU Lesser General Public License version 2.1
?>
<?php

/**

	When using MYSQL Stored Procedures, user code can either call a
	stored procedure, which returns a results set;
	or they can call a stored function, which returns a single value.

	function getLastLogin( $sid, $USER )
	{
		$result = ReplicantDB::CallFunction( DB, "Users_Last_Login( '$sid', '$USER' )", $debug );

		if ( "OK" = $result->status )
		{
			return $result->value;
		}
		else
		{
			return FALSE; // ???
		}
	}
	
	function getUsers( $sid )
	{
		$result = ReplicantDB::CallQuery( DB, "SELECT * FROM users", $debug );

		if ( "OK" = $result->status )
		{
			return $result->results;
		}
		else
		{
			return FALSE; // ???
		}
	}
	
	function updateUserName( $sid, $USER, $given_name, $family_name )
	{
		$result = ReplicantDB::CallProcedure( DB, "Users_Update_Name( '$sid', '$USER', '$given_name', '$family_name' )", $debug );
	
		if ( "OK" == $result->status )
		{
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
 */

namespace replicantdb;

class ReplicantDB
{
	private static $only      = null;
	private static $primary   = null;
	private static $secondary = null;

	private $username = "";
	private $password = "";
	private $hostname = "";
	private $message  = "";
	private $error    = "";

	static function ContainsDatabase( $database, $debug )
	{
		$sql = "SHOW DATABASES";

		$result = array();
		$result['error'   ]  = "";
		$result['failover']  = "FALSE";
		$result['hostname']  = "";
		$result['message' ]  = "";
		$result['status'  ]  = "ERROR";
		$result['results' ]  = FALSE;
		$result['warning' ]  = "";
		$result['URL']       = REDIRECT_URL;
		$result['target_id'] = array_key_exists( "target_id", $_REQUEST ) ? \Input::Filter( $_REQUEST["target_id"] ) : "";
		$result['submit']    = array_key_exists(    "submit", $_REQUEST ) ? \Input::Filter( $_REQUEST["submit"]    ) : "";
		$result['microsecs'] = 0;

		$db0 = self::Singleton( "ONLY"      );
		$db1 = self::Singleton( "PRIMARY"   );
		$db2 = self::Singleton( "SECONDARY" );

		$return_as_float = TRUE;

		$start = microtime( $return_as_float );

		self::TryInfo( $result, $db0,  "DB_HOSTNAME", $sql, $debug ) ||
		self::TryInfo( $result, $db1, "DB_HOSTNAME1", $sql, $debug ) ||
		self::TryInfo( $result, $db2, "DB_HOSTNAME2", $sql, $debug, true );

		if ( FALSE !== $result['results'] )
		{
			$result["status"] = "OK";
		}
		else
		if ( ! IsCLI() )
		{
			$result['results'] = null;

			if ( string_contains( $result['error'], "does not exist" ) )
			{
				header( ':', true, 501 );
			
				//	Not available on earlier version of PHP.
				//
				//\http_response_code( 501 );
			}

			//error_log( $result['error'] );
		}

		$conversion_from_seconds_to_microseconds = 1000000;

		$result['microsecs'] = (int) ((microtime( $return_as_float ) - $start) * $conversion_from_seconds_to_microseconds);
		
		return (object) $result;
	}

	static function CallInfo( $sql, $debug )
	{
		$result = array();
		$result['error'   ]  = "";
		$result['failover']  = "FALSE";
		$result['hostname']  = "";
		$result['message' ]  = "";
		$result['status'  ]  = "ERROR";
		$result['results' ]  = FALSE;
		$result['warning' ]  = "";
		$result['URL']       = REDIRECT_URL;
		$result['target_id'] = array_key_exists( "target_id", $_REQUEST ) ? \Input::Filter( $_REQUEST["target_id"] ) : "";
		$result['submit']    = array_key_exists(    "submit", $_REQUEST ) ? \Input::Filter( $_REQUEST["submit"]    ) : "";
		$result['microsecs'] = 0;

		$db0 = self::Singleton( "ONLY"      );
		$db1 = self::Singleton( "PRIMARY"   );
		$db2 = self::Singleton( "SECONDARY" );

		$return_as_float = TRUE;

		$start = microtime( $return_as_float );

		self::TryInfo( $result, $db0, "DB_HOSTNAME",  $sql, $debug ) ||
		self::TryInfo( $result, $db1, "DB_HOSTNAME1", $sql, $debug ) ||
		self::TryInfo( $result, $db2, "DB_HOSTNAME2", $sql, $debug, true );

		if ( FALSE !== $result['results'] )
		{
			$result["status"] = "OK";
		}
		else
		if ( ! IsCLI() )
		{
			$result['results'] = null;

			if ( string_contains( $result['error'], "does not exist" ) )
			{
				header( ':', true, 501 );
			
				//	Not available on earlier version of PHP.
				//
				//\http_response_code( 501 );
			}

			//error_log( $result['error'] );
		}

		$conversion_from_seconds_to_microseconds = 1000000;

		$result['microsecs'] = (int) ((microtime( $return_as_float ) - $start) * $conversion_from_seconds_to_microseconds);
		
		return (object) $result;
	}

	static function CallProcedure( $db_name, $procedure, $debug, $db_version = DB_VERSION )
	{
		$command = self::ExtractCommand( $procedure );

		$database = $db_name . $db_version;

		if ( string_contains( $command, "Replace" ) || string_contains( $command, "Update" ) || string_contains( $command, "Create" ) || string_contains( $command, "Archive" ) )
		{
			return self::CallLoggedProcedure( $database, $procedure, $debug );
		}
		else
		{
			return self::CallUnloggedProcedure( $database, $procedure, $debug );
		}
	}

	static function CallLoggedProcedure( $database, $procedure, $debug )
	{
		$result  = self::PrivateCallProcedure( $database, $procedure, $debug );
		$result2 = self::_LogProcedure( $database, $procedure, $result, $debug );

		return (object) $result;
	}

	static function CallUnloggedProcedure( $database, $procedure, $debug )
	{
		return self::PrivateCallProcedure( $database, $procedure, $debug );
	}

	static function _LogProcedure( $database, $procedure, $result, $debug )
	{
		$sid       = self::ExtractSid( $procedure );
		$encoded   = htmlentities( $procedure, ENT_QUOTES );
		$type      = "???";
		$microsecs = $result->microsecs;
		$status    = $result->status;
		$result    = "";
		$error     = property_exists( $result, "error" ) ? $result->error : "";

		$ipaddress = array_get( $_SERVER, "REMOTE_ADDR"  );
		$referrer  = array_get( $_SERVER, "HTTP_REFERER" );

		$sql     = "ReplicantDB_Log_Replace( '$sid', '$encoded', '$type', '$microsecs', '$status', '$result', '$error', '$ipaddress', '$referrer' )";

		return self::PrivateCallProcedure( $database, $sql, $debug );
	}

	static function PrivateCallProcedure( $database, $procedure, $debug )
	{
		$sql = "CALL $procedure";

		$result = array();
		$result['error'   ]  = "";
		$result['failover']  = "FALSE";
		$result['hostname']  = "";
		$result['message' ]  = "";
		$result['status'  ]  = "ERROR";
		$result['results' ]  = FALSE;
		$result['warning' ]  = "";
		$result['URL']       = REDIRECT_URL;
		$result['target_id'] = array_key_exists( "target_id", $_REQUEST ) ? \Input::Filter( $_REQUEST["target_id"] ) : "";
		$result['submit']    = array_key_exists(    "submit", $_REQUEST ) ? \Input::Filter( $_REQUEST["submit"]    ) : "";
		$result['limit']     = array_key_exists(     "limit", $_REQUEST ) ? \Input::Filter( $_REQUEST["limit"]     ) : "";
		$result['offset']    = array_key_exists(    "offset", $_REQUEST ) ? \Input::Filter( $_REQUEST["offset"]    ) : "";
		$result['microsecs'] = 0;

		$db0 = self::Singleton( "ONLY"      );
		$db1 = self::Singleton( "PRIMARY"   );
		$db2 = self::Singleton( "SECONDARY" );

		$return_as_float = TRUE;

		$start = microtime( $return_as_float );

		self::TryProcedure( $result, $db0, $database,  "DB_HOSTNAME", $sql, $debug ) ||
		self::TryProcedure( $result, $db1, $database, "DB_HOSTNAME1", $sql, $debug ) ||
		self::TryProcedure( $result, $db2, $database, "DB_HOSTNAME2", $sql, $debug, true );

		if ( FALSE !== $result['results'] )
		{
			$result["status"] = "OK";
		}
		else
		if ( ! IsCLI() )
		{
			$result['results'] = null;

			if ( string_contains( $result['error'], "does not exist" ) )
			{
				header( ':', true, 501 );
			
				//	Not available on earlier version of PHP.
				//
				//\http_response_code( 501 );
			}

			//error_log( $result['error'] );
		}

		$conversion_from_seconds_to_microseconds = 1000000;

		$result['microsecs'] = (int) ((microtime( $return_as_float ) - $start) * $conversion_from_seconds_to_microseconds);
		
		return (object) $result;
	}

	static function CallFunction( $db_name, $function, $debug, $db_version = DB_VERSION )
	{
		$database = $db_name . $db_version;

		$sql = "SELECT $function";

		$result = array();
		$result['status'  ] = "ERROR";
		$result['message' ] = "";
		$result['warning' ] = "";
		$result['hostname'] = "";

		$result['error'   ] = "";
		$result['failover'] = "FALSE";
		$result['value'   ] = FALSE;

		$db0 = self::Singleton( "ONLY"      );
		$db1 = self::Singleton( "PRIMARY"   );
		$db2 = self::Singleton( "SECONDARY" );

		self::TryFunction( $result, $db0, $database,  "DB_HOSTNAME", $sql, $debug ) ||
		self::TryFunction( $result, $db1, $database, "DB_HOSTNAME1", $sql, $debug ) ||
		self::TryFunction( $result, $db2, $database, "DB_HOSTNAME2", $sql, $debug, true );
		
		if ( FALSE !== $result['value'] )
		{
			$result["status"] = "OK";
		}
		else
		{
			http_response_code( 503 );
		}
		
		return (object) $result;
	}

	static function TryProcedure( &$result, $db, $database, $const, $sql, $debug, $failover = False )
	{
		$connection = False;

		if ( (False === $result['results']) && $db )
		{
			$result['hostname'] = $db->hostname . " ($const)";
			if ( $failover )
			{
				$result['failover'] = "TRUE";
				$result['warning']  = "Attempting failover to secondary";
			}

			if ( ($connection = $db->connect( $debug )) )
			{
				$result['results'] = $db->_callProcedure( $database, $sql, $debug );

				if ( "READ_ONLY" == $db->error ) $connection = FALSE;
			}
			$result['message'] = $db->message; $db->message = "";
			$result['error'  ] = $db->error;   $db->error   = "";
		}

		return $connection;
	}

	static function TryFunction( &$result, $db, $database, $const, $sql, $debug, $failover = False )
	{
		$connection = False;

		if ( (False === $result['value']) && $db )
		{
			$result['hostname'] = $db->hostname . " ($const)";
			if ( $failover )
			{
				$result['failover'] = "TRUE";
				$result['warning']  = "Attempting failover to secondary";
			}

			if ( ($connection = $db->connect( $debug )) )
			{
				$result['value'] = $db->_callFunction( $database, $sql, $debug );

				if ( "READ_ONLY" == $db->error ) $connection = FALSE;
			}
			$result['message'] = $db->message; $db->message = "";
			$result['error'  ] = $db->error;   $db->error   = "";
		}
		return $connection;
	}

	static function TryInfo( &$result, $db, $const, $sql, $debug, $failover = False )
	{
		$connection = False;

		if ( (False === $result['results']) && $db )
		{
			$result['hostname'] = $db->hostname . " ($const)";
			if ( $failover )
			{
				$result['failover'] = "TRUE";
				$result['warning']  = "Attempting failover to secondary";
			}

			if ( ($connection = $db->connect( $debug )) )
			{
				$result['results'] = $db->_info( $sql );

				if ( "READ_ONLY" == $db->error ) $connection = FALSE;
			}
			$result['message'] = $db->message; $db->message = "";
			$result['error'  ] = $db->error;   $db->error   = "";
		}

		return $connection;
	}

	static function Singleton( $type )
	{
		if ( defined( "DB_USERNAME" ) && defined( "DB_PASSWORD" ) )
		{
			switch ( $type )
			{
			case "ONLY":
				if ( ! self::$only && defined( "DB_HOSTNAME" ) )
				{
					self::$only = new ReplicantDB( DB_USERNAME, DB_HOSTNAME, DB_PASSWORD );
				}
				return self::$only;

			case "PRIMARY":
				if ( ! self::$primary && defined( "DB_HOSTNAME1" ) )
				{
					$username = DB_USERNAME;
					$password = DB_PASSWORD;
					$hostname = DB_HOSTNAME1;

					self::$primary = new ReplicantDB( DB_USERNAME, DB_HOSTNAME1, DB_PASSWORD );
				}
				return self::$primary;
			
			case "SECONDARY":
				if ( ! self::$secondary && defined( "DB_HOSTNAME2" ) )
				{
					$username = DB_USERNAME;
					$password = DB_PASSWORD;
					$hostname = DB_HOSTNAME2;

					self::$secondary = new ReplicantDB( DB_USERNAME, DB_HOSTNAME2, DB_PASSWORD );
				}
				return self::$secondary;
			}
		}
		else
		{
			return null;
		}
	}

	function __construct( $username, $hostname, $password )
	{
		$this->username = $username;
		$this->hostname = $hostname;
		$this->password = $password;
	}

	function connect( $debug )
	{
		$user = $this->username;
		$host = $this->hostname;
		$pass = $this->password;

		$this->mysqli = mysqli_init();

		if ( $this->mysqli )
		{
			if ( file_exists( "/local/settings/services/mysql/ssl/$host/client-key.pem" ) )
			{
				mysqli_ssl_set
				(
					$this->mysqli,
					"/local/settings/services/mysql/ssl/$host/client-key.pem",
					"/local/settings/services/mysql/ssl/$host/client-cert.pem",
					"/local/settings/services/mysql/ssl/$host/ca-cert.pem",
					NULL,
					NULL
				);
			}
			$this->mysqli->real_connect( $host, $user, $pass );
		
			if ( NULL == $this->mysqli->connect_error )
			{
				return True;
			} else {
				$this->message = "Could not connect to $user@$host:$pass!!!";
				$this->mysqli  = NULL;
				return False;
			}
		}
		else
		{
			$this->message = "Could not initialise mysqli!!!";

			return False;
		}
	}

	function _callProcedure( $database, $sql_query, $debug )
	{
		$tuples = False;

		if ( mysqli_select_db( $this->mysqli, $database ) )
		{
			$resource = mysqli_query( $this->mysqli, $sql_query );
			if ( True === $resource )
			{
				$this->message = "SQL: $sql_query : (TRUE)";

				$tuples = array();
			}
			else if ( False === $resource )
			{
				$this->error   = $this->lastErrorMessage();
				$this->message = "SQL: $sql_query : (FALSE) - " . $this->error;

				$tuples = False;
			}
			else if ( $resource )
			{
				$nr_results = 0;

				$tuples = array();
				while ( $row = mysqli_fetch_array( $resource, MYSQLI_ASSOC ) )
				{
					if ( true )
					{
						$tuples[] = (object) $row;
					}
					else
					{
						$tuples[] = self::DecodeHTMLEntities( $row );
					}
				}

				mysqli_free_result( $resource );
			}
			else
			{
				$this->message = "Error in SQL: $sql_query - " . $this->lastErrorMessage();
			}
		}
		else
		{
			$this->message = "Could not select database: $this->username@$this->hostname/$database";
		}

		return $tuples;
	}

	function _info( $sql_query )
	{
		$tuples = array();
		$resource = mysqli_query( $this->mysqli, $sql_query );
		if ( $resource )
		{
			while ( $row = mysqli_fetch_array( $resource, MYSQLI_ASSOC ) )
			{
				$tuples[] = $row;
			}
		} else {
			error_log( $this->lastErrorMessage() );

			echo "<!-- " . $this->lastErrorMessage() . " -->";
		}
		return $tuples;
	}

	static function DecodeHTMLEntities( $tuple )
	{
		$row = array();
	
		foreach ( $tuple as $key => $val )
		{
			$decoded_key = "!!" . $key;
			$decoded_val = html_entity_decode( $val );
		
			$row[$decoded_key] = $decoded_val;
			$row[$key]         = $val;
		}
		
		return (object) $row;
	}

	function _callFunction( $database, $sql_query, $debug )
	{
		$ret = False;

		if ( mysqli_select_db( $this->mysqli, $database ) )
		{
			$resource = mysqli_query( $this->mysqli, $sql_query );
			if ( True === $resource )
			{
				$this->message = "SQL: $sql_query : (TRUE)";
			
				$ret = True;
			}
			else if ( False === $resource )
			{
				$this->error   = $this->lastErrorMessage();
				$this->message = "SQL: $sql_query : (FALSE) - " . $this->error;

				$ret = False;
			}
			else if ( $resource )
			{
				$nr_results = 0;

				while ( $row = mysqli_fetch_array( $resource, MYSQLI_NUM ) )
				{
					$ret = $row[0];
				}

				$this->message = "SQL: $sql_query : ($ret)";

				mysqli_free_result( $resource );
			}
			else
			{
				$this->message = "Error in SQL: $sql_query - " . $this->lastErrorMessage();
			}
		}
		else
		{
			$this->message = "Could not select database: $this->username@$this->hostname/$database";
		}

		return $ret;
	}

	function lastErrorMessage()
	{
		return mysqli_error( $this->mysqli );
	}

	static function ExtractCommand( $procedure )
	{
		$cmd  = "";
		$bits = explode( "(", $procedure );

		if ( 0 < count( $bits ) )
		{
			$cmd = $bits[0];
		}

		return $cmd;
	}

	static function ExtractSid( $procedure )
	{
		$sid  = "";
		$bits = explode( "'", $procedure );

		if ( (2 < count( $bits )) && (64 == strlen( $bits[1] )) )
		{
			$sid = $bits[1];
		}

		return $sid;
	}
}


/*

Modified from:
http://stackoverflow.com/questions/933367/php-how-to-best-determine-if-the-current-invocation-is-from-cli-or-web-server
*/

function IsCLI()
{
    return (php_sapi_name() === 'cli');
}
