<?php

class ReplicantAPIPage extends \APIPage
{
	function __construct()
	{
		parent::__construct();
	}

	function render()
	{
		if ( "/api/multiselect/" == REDIRECT_URL )
		{
			\JSON::EchoStream( \Selects::RetrieveMulti( $this->getSessionID(), $this->request, $this->debug ) );
		}
		else
		{
			$command  = $this->getRequest( "command" );

			$trio = self::DetermineProcedureNameFromEither( $command, REDIRECT_URL );

			$proc   = $trio["procedure"];
			$suffix = $trio["suffix"];
			$is_csv = $trio["is_csv"];
			$map    = $is_csv ? self::GenerateMap( array_get( $this->request, "csv_map" ) ) : null;

			if ( ! $proc )
			{
				error_log( "Could not determine procedure name from either: '$command' or '" . REDIRECT_URL . "'" );

				echo '{"status":"ERROR","error":"Could not determine procedure"}';
			}
			else
			if ( ! defined( "DB_VERSION" ) )
			{
				error_log( "DB version not identified !!!" );
			}
			else
			{
				$sid = $this->getSessionId();

				$sql = self::DetermineSQLMappingForProcedure( $sid, $proc, $suffix, $this->request, $this->debug );

				if ( ! $sql )
				{
					parent::render();
				}
				else
				{
					$result = \replicantdb\ReplicantDB::CallProcedure( DB, $sql, $this->debug );

					error_log( DB . DB_VERSION . "@" . DB_HOSTNAME . ":" . $sql . " : " . $result->status . ", " . $result->error );

					if ( ! $is_csv )
					{
						header( "Content-Type: application/json" );

						echo \JSON::Encode( $result );
					}
					else
					if ( "OK" == $result->status )
					{
						$content = "";

						$ds = ("true" == $this->getRequest( "double_space" ));

						if ( $map )
						{
							error_log( "Calling CSV::encode with mapped tuples." );
						
							$content = \CSV::encode( \CSV::MapValues( $map, $result->results ), true, $ds );
						}
						else
						{
							error_log( "Calling CSV::encode with unmapped tuples." );

							$content = \CSV::encode( $result->results, true, $ds );
						}
						$size = strlen( $content );

						header( "Content-Type: application/octet-stream" );
						header( "Content-Disposition: attachment;filename=\"download.csv\"" );
						header( "Content-Length: $size" );
       				 	//header( "Pragma: public" );
				        //header( "Cache-Control: must-revalidate, post-check=0, pre-check=0" );

        				echo $content;

		                ob_flush();
       			        flush();
						exit;
					}
				}
			}
		}
	}

	static function DetermineProcedureNameFromEither( $command, $redirect_url )
	{
		$proc   =    "";
		$suffix =    "";
		$is_csv = false;

		if ( "" != $command )
		{
			$proc = $command;
		}
		else
		{
			$parts = explode( " ", trim( str_replace( '/', ' ', $redirect_url ) ) );
			$some  = array_slice( $parts, 1 );
			$proc  = implode( "_", $some );

			switch ( $parts[0] )
			{
			case "api":
				break;
				
			default:
				$suffix = ("_" . $parts[0]);
			}
		}

		if ( self::StringEndsWith( $proc, "_csv" ) )
		{
			$is_csv = true;

			$proc = substr( $proc, 0, strlen( $proc ) - 4 );
		}
	
		return array( "procedure" => $proc, "suffix" => $suffix, "is_csv" => $is_csv );
	}

	static function DetermineSQLMappingForProcedure( $sid, $proc, $suffix, $request, $debug )
	{
		$mapping = null;

		$database = DB . DB_VERSION;

		$proc1 = $proc . $suffix;
		$proc2 = $proc . "_retrieve" . $suffix;
		$procX = $proc1;

		$sql = "Retrieve_Parameters_For( '$database', '$proc1' )";
		
		$result = \replicantdb\ReplicantDB::CallFunction( DB, $sql, $debug );

		if ( defined( "VERBOSE" ) && VERBOSE ) error_log( $sql . " : " . $result->error );

		if ( null == $result->value )
		{
			$sql = "Retrieve_Parameters_For( '$database', '$proc2' )";
		
			$result = \replicantdb\ReplicantDB::CallFunction( DB, $sql, $debug );

			if ( defined( "VERBOSE" ) && VERBOSE ) error_log( $sql . " : " . $result->error );

			$procX = $proc2;
		}

        if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "Request: " . var_export( $request, true ) );
	
		if ( null == $result->value )
		{
			error_log( "Could not find procedure for: '$proc1' or '$proc2'" );
		}
		else
		{
			$arguments = "( ";
			$parameters = self::ConvertParametersToDictionary( $result->value );

			foreach ( $parameters as $parameter )
			{
				// Strip of leading '$' and make lowercase.
				$parameter_name = str_replace( '$', '', $parameter );
				$value          = array_get( $request, $parameter_name );

				if ( ! $value ) $value = array_get( $request, strtolower( $parameter_name ) );

				$arguments     .= ("'" . $value . "', ");

                if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "$parameter_name ($parameter) --> $value" );
			}
			
			$arguments = substr( $arguments, 0, -2 );
			
			$arguments .= " )";

			$mapping = $procX . $arguments;
		}
		return $mapping;
	}

	static function ConvertParametersToDictionary( $string )
	{
		$parameters = array();
		$bits       = explode( ",", trim( $string ) );
		
		foreach ( $bits as $parameter )
		{
			$parts = explode( " ", trim( $parameter ) );
			
			$parameters[] = $parts[0];

			if ( defined( "VERBOSE" ) && VERBOSE ) error_log( $parts[0] . " " . $parts[1] );
		}
		
		return $parameters;
	}

	static function GenerateMap( $base64_encoded_csv_map )
	{
		$csv_map =  base64_decode( $base64_encoded_csv_map );
	
		if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "GenerateMap( \"$csv_map\" )" );
	
		//
		//	csv_map=Family+Name|family_name,...
		//
	
		$map = null;

		if ( $csv_map )
		{
			$map  = array();
			$bits = explode( ",", $csv_map );
			
			foreach ( $bits as $mapping )
			{
				$pieces = explode( "|", $mapping );

				if ( 2 == count( $pieces ) )
				{
					$key = urldecode( $pieces[0] );
					$val = urldecode( $pieces[1] );
				
					$map[$key] = $val;
				}

				if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "Mapping( $mapping ) : $key | $val" );
			}
		}
		return $map;
	}

	static function StringEndsWith( $string, $suffix )
	{
		$str1 = str_replace( $suffix, '', $string );
		$str2 = substr( $string, 0, strlen( $string ) - strlen( $suffix ) );

		if ( $str1 == $str2 )
		{
			if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "StringEndsWith( \"$string\", \"$suffix\" ) : TRUE" );
		}
		else
		{
			if ( defined( "VERBOSE" ) && VERBOSE ) error_log( "StringEndsWith( \"$string\", \"$suffix\" ) : FALSE" );

			if ( defined( "VERBOSE" ) && VERBOSE ) error_log( $str1 );
			if ( defined( "VERBOSE" ) && VERBOSE ) error_log( $str2 );
		}
		
		return ( $str1 == $str2 );
	}
}
